###input data file
indat<-read.csv(file='R:/teaching/short course/missing data/session2/Empirical study/nhanes.subset.csv')
n<-dim(indat)[1]

###create missingness
attach(indat)
phi<-c(0.5,-0.5,-0.5)
X1<-(RIDAGEYR-mean(RIDAGEYR))/sd(RIDAGEYR)
X2<-(BMXBMI-mean(BMXBMI))/sd(BMXBMI)
p<-exp(phi[1]+phi[2]*X1+phi[3]*X2)/(1+exp(phi[1]+phi[2]*X1+phi[3]*X2))
r<-rbinom(n,1,p)
Y<-BPXSY1
Y[r==0]<-NA
indat2<-cbind(BPXSY1,X1,X2,r)
write.csv(indat2,file='R:/teaching/short course/missing data/session2/Empirical study/outdat.csv')

###0.full sample estimators:
est1<-mean(BPXSY1)
est2<-median(BPXSY1)
est<-c(est1,est2)

###create cell variable
X1_c<-rep(0,n)
X2_c<-rep(0,n)
q1<-quantile(X1,probs=seq(0,1,0.25))[2:4]
q2<-quantile(X2,probs=seq(0,1,0.25))[2:4]

X1_c[X1<q1[1]]<-1
X1_c[X1>=q1[1] & X1<q1[2]]<-2
X1_c[X1>=q1[2] & X1<q1[3]]<-3
X1_c[X1>=q1[3]]<-4

X2_c[X2<q2[1]]<-1
X2_c[X2>=q2[1] & X2<q2[2]]<-2
X2_c[X2>=q2[2] & X2<q2[3]]<-3
X2_c[X2>=q2[3]]<-4

cell<-paste(X1_c,X2_c,sep='')
cell_name<-names(table(cell))
n_cell<-length(cell_name)

###1.Simple imputation by using cell mean
IY_Sim_Mean<-Y
for(i in 1:n_cell){
IY_Sim_Mean[is.na(Y)==T & cell==cell_name[i]]<-mean(Y[is.na(Y)==F & cell==cell_name[i]])
}

est1_Sim_Mean<-mean(IY_Sim_Mean)
est2_Sim_Mean<-median(IY_Sim_Mean)
est_Sim_Mean<-c(est1_Sim_Mean,est2_Sim_Mean)

###2.Parametric imputation with normal distribution for y|X1,X2
Yr<-Y[r==1]
X1r<-X1[r==1]
X2r<-X2[r==1]
nr<-length(Yr)
nm<-n-nr
fit<-lm(Yr~X1r+X2r)
ebeta<-fit$coefficients
esigma<-sqrt(sum(fit$residuals^{2})/nr)
r_e<-rnorm(nm,mean=0,sd=esigma)
IY_Par<-Y
IY_Par[is.na(Y)==T]<-ebeta[1]+ebeta[2]*X1[is.na(Y)==T]+ebeta[3]*X2[is.na(Y)==T]+r_e

est1_Par<-mean(IY_Par)
est2_Par<-median(IY_Par)
est_Par<-c(est1_Par,est2_Par)

###3.Random Hot-deck imputation
IY_HD<-Y
for(i in 1:n_cell){
nm_cell<-length(Y[is.na(Y)==T & cell==cell_name[i]])
IY_cell<-sample(Y[is.na(Y)==F & cell==cell_name[i]],nm_cell,replace=T)
IY_HD[is.na(Y)==T & cell==cell_name[i]]<-IY_cell
}

est1_HD<-mean(IY_HD)
est2_HD<-median(IY_HD)
est_HD<-c(est1_HD,est2_HD)

###4.Nearest neighbor imputation (predicted mean matching)
library(mice)
RY1<-rep(1,n)
RY1[is.na(Y)==T]<-0
RY<-(RY1==1)
IY_PMM<-Y
imp<-mice.impute.pmm(IY_PMM,RY,cbind(X1,X2),donors=5,type=1)
IY_PMM2<-c(Y[RY],imp)

est1_PMM<-mean(IY_PMM2)
est2_PMM<-median(IY_PMM2)
est_PMM<-c(est1_PMM,est2_PMM)

###5.Regression imputation

#5.1. deterministic regression imputation
IY_DREG<-Y
IY_DREG[is.na(Y)==T]<-ebeta[1]+ebeta[2]*X1[is.na(Y)==T]+ebeta[3]*X2[is.na(Y)==T]
est1_DREG<-mean(IY_DREG)
est2_DREG<-median(IY_DREG)
est_DREG<-c(est1_DREG,est2_DREG)

#5.2. stochastic regression imputation
resd_r<-fit$residuals/esigma
resd_m<-sample(resd_r,nm,replace=T)
IY_SREG<-Y
IY_SREG[is.na(Y)==T]<-ebeta[1]+ebeta[2]*X1[is.na(Y)==T]+ebeta[3]*X2[is.na(Y)==T]+esigma*resd_m
est1_SREG<-mean(IY_SREG)
est2_SREG<-median(IY_SREG)
est_SREG<-c(est1_SREG,est2_SREG)

#5.3. Bayesian linear regression imputation
IY_BREG<-Y
imp<-mice.impute.norm(IY_BREG,RY,cbind(X1,X2))
IY_BREG2<-c(Y[RY],imp)

est1_BREG<-mean(IY_BREG2)
est2_BREG<-median(IY_BREG2)
est_BREG<-c(est1_BREG,est2_BREG)

###6. Multiple imputation
IY_MI<-Y
XY<-cbind(IY_MI,X1,X2)
MI<-mice(XY,m=5,method=c('norm'))
imp<-MI$imp[[1]]

#create repeated data files
dat_r<-matrix(rep(Y[is.na(Y)==F],5),nr,5)
colnames(dat_r)<-colnames(imp)
dat_rep<-rbind(dat_r,imp)
Mean_rep<-apply(dat_rep,2,mean)
Median_rep<-apply(dat_rep,2,median)

est1_MI<-mean(Mean_rep)
est2_MI<-mean(Median_rep)
est_MI<-c(est1_MI,est2_MI)




